==> bafser-1.6.tar.gz: bafser-1.6/src/bafser/data/log.py <==
from datetime import datetime
from typing import Any, Union

from sqlalchemy import Column, DateTime, orm, Integer, String, JSON
from sqlalchemy.orm import Session

from .. import SqlAlchemyBase, TableBase, UserBase, IdMixin, get_datetime_now

FieldName = str
NewValue = Any
OldValue = Any


class Log(SqlAlchemyBase, IdMixin):
    __tablename__ = "Log"

    date = Column(DateTime, nullable=False)
    actionCode = Column(String(16), nullable=False)
    userId = Column(Integer, nullable=False)
    userName = Column(String(64), nullable=False)
    tableName = Column(String(16), nullable=False)
    recordId = Column(Integer, nullable=False)
    changes = Column(JSON, nullable=False)

    def __repr__(self):
        return f"<Log> [{self.id}] {self.date} {self.actionCode}"

    def get_dict(self):
        return self.to_dict(only=("id", "date", "actionCode", "userId", "userName", "tableName", "recordId", "changes"))

    @staticmethod
    def added(
        record: TableBase,
        actor: Union[UserBase, None],
        changes: list[tuple[FieldName, NewValue]],
        now: datetime = None,
        commit=True,
        db_sess: Session = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.added,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=-1,
            changes=list(map(lambda v: (v[0], None, v[1]), changes))
        )
        db_sess.add(log)
        if isinstance(record, IdMixin):
            if record.id is not None:
                log.recordId = record.id
            elif commit:
                db_sess.commit()
                log.recordId = record.id
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def updated(
        record: TableBase,
        actor: Union[UserBase, None],
        changes: list[tuple[FieldName, OldValue, NewValue]],
        now: datetime = None,
        commit=True,
        db_sess: Session = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.updated,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=changes
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def deleted(
        record: TableBase,
        actor: Union[UserBase, None],
        changes: list[tuple[FieldName, OldValue]] = [],
        now: datetime = None,
        commit=True,
        db_sess: Session = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.deleted,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=list(map(lambda v: (v[0], v[1], None), changes))
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def restored(
        record: TableBase,
        actor: Union[UserBase, None],
        changes: list[tuple[FieldName, OldValue, NewValue]] = [],
        now: datetime = None,
        commit=True,
        db_sess: Session = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.restored,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=changes
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log


class Actions:
    added = "added"
    updated = "updated"
    deleted = "deleted"
    restored = "restored"

==> bafser-2.0.5.tar.gz: bafser-2.0.5/src/bafser/data/log.py <==
from datetime import datetime
from typing import Any

from sqlalchemy import String, JSON
from sqlalchemy.orm import Session, Mapped, mapped_column

from .. import SqlAlchemyBase, UserBase, IdMixin, get_datetime_now

FieldName = str
NewValue = Any
OldValue = Any
Changes = list[tuple[FieldName, OldValue, NewValue]]


class Actions:
    added = "added"
    updated = "updated"
    deleted = "deleted"
    restored = "restored"


class Log(SqlAlchemyBase, IdMixin):
    __tablename__ = "Log"

    date: Mapped[datetime]
    actionCode: Mapped[str] = mapped_column(String(16))
    userId: Mapped[int]
    userName: Mapped[str] = mapped_column(String(64))
    tableName: Mapped[str] = mapped_column(String(16))
    recordId: Mapped[int]
    changes: Mapped[Changes] = mapped_column(JSON)

    def __repr__(self):
        return f"<Log> [{self.id}] {self.date} {self.actionCode}"

    def get_dict(self):
        return self.to_dict(only=("id", "date", "actionCode", "userId", "userName", "tableName", "recordId", "changes"))

    @staticmethod
    def added(
        record: SqlAlchemyBase,
        actor: UserBase | None,
        changes: list[tuple[FieldName, NewValue]],
        now: datetime | None = None,
        commit: bool = True,
        db_sess: Session | None = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        assert db_sess
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.added,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=-1,
            changes=list(map(lambda v: (v[0], None, v[1]), changes))
        )
        db_sess.add(log)
        if isinstance(record, IdMixin):
            if record.id is not None:  # type: ignore
                log.recordId = record.id
            elif commit:
                db_sess.commit()
                log.recordId = record.id
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def updated(
        record: SqlAlchemyBase,
        actor: UserBase | None,
        changes: Changes,
        now: datetime | None = None,
        commit: bool = True,
        db_sess: Session | None = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        assert db_sess
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.updated,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=changes
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def deleted(
        record: SqlAlchemyBase,
        actor: UserBase | None,
        changes: list[tuple[FieldName, OldValue]] = [],
        now: datetime | None = None,
        commit: bool = True,
        db_sess: Session | None = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        assert db_sess
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.deleted,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=list(map(lambda v: (v[0], v[1], None), changes))
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log

    @staticmethod
    def restored(
        record: SqlAlchemyBase,
        actor: UserBase | None,
        changes: Changes = [],
        now: datetime | None = None,
        commit: bool = True,
        db_sess: Session | None = None,
    ):
        if actor is None:
            actor = UserBase.get_fake_system()
        db_sess = db_sess if db_sess else Session.object_session(actor)
        assert db_sess
        if now is None:
            now = get_datetime_now()
        log = Log(
            date=now,
            actionCode=Actions.restored,
            userId=actor.id,
            userName=actor.name,
            tableName=record.__tablename__,
            recordId=record.id if isinstance(record, IdMixin) else -1,
            changes=changes
        )
        db_sess.add(log)
        if commit:
            db_sess.commit()
        return log
